// Copyright (C) 2000 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "bayonnedb.h"
#include <getopt.h>

Keys::Keys(unsigned ks) :
Mutex()
{
	keysize = ks;
	map = new key_t*[keysize];
	memset(map, 0, sizeof(map) * keysize);
}

Keys::~Keys()
{
	if(map)
		delete[] map;
}

unsigned Keys::getPath(const char *str)
{
	unsigned value = 0;

	while(*str)
		value = (value << 1) ^ (*(str++) & 0x1f);
	
	return value % keysize;
}

void Keys::del(unsigned path, key_t *key)
{
	if(key->next)
		key->next->prev = key->prev;
	if(key->prev)
		key->prev->next = key->next;
	if(map[path] == key)
		map[path] = key->next;
	delete[] key;
}

void Keys::add(const char *str, unsigned exp, unsigned len)
{
	key_t *key = (key_t *)new char[sizeof(key_t) + strlen(str)];
	unsigned short path = getPath(str);

	if(!len)
		len = strlen(str);
	
	time(&key->expires);
	key->expires += exp;
	strcpy(key->text, str);
	key->next = key->prev = NULL;
	key->len = len;
	EnterMutex();
	if(map[path])
		map[path]->prev = key;
	key->next = map[path];
	map[path] = key;
	LeaveMutex();
}

char *Keys::find(const char *str)
{
	time_t now;
	key_t *link, *next;
	unsigned path = getPath(str);
	EnterMutex();
	link = map[path];

	time(&now);
	while(link)
	{
		next = link->next;
		if(now > link->expires)
			del(path, link);
		else if(!strnicmp(str, link->text, link->len))
		{
			LeaveMutex();
			return link->text + link->len;
		}
		link = next;
	}
	LeaveMutex();
	return NULL;	
}

void Keys::expire(void)
{
	unsigned path;
	key_t *key, *next;
	time_t now;

	time(&now);
	for(path = 0; path < keysize; ++ path)
	{
		key = map[path];
		while(key)
		{
			next = key->next;
			if(now > key->expires)
				del(path, key);
			key = next;
		}
	}
}


















